<?php

/**
* Copyright Maarch since 2008 under licence GPLv3.
* See LICENCE.txt file at the root folder for more details.
* This file is part of Maarch software.
*
*/

/**
 * @brief Attachment Model Abstract
 * @author dev@maarch.org
 */

namespace Attachment\models;

use SrcCore\models\CoreConfigModel;
use SrcCore\models\DatabaseModel;
use SrcCore\models\ValidatorModel;

abstract class AttachmentModelAbstract
{
    public static function get(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['select']);
        ValidatorModel::arrayType($aArgs, ['select', 'where', 'data', 'orderBy', 'groupBy']);
        ValidatorModel::intType($aArgs, ['limit']);

        $attachments = DatabaseModel::select([
            'select'    => $aArgs['select'],
            'table'     => ['res_attachments'],
            'where'     => empty($aArgs['where']) ? [] : $aArgs['where'],
            'data'      => empty($aArgs['data']) ? [] : $aArgs['data'],
            'order_by'  => empty($aArgs['orderBy']) ? [] : $aArgs['orderBy'],
            'groupBy'   => empty($aArgs['groupBy']) ? [] : $aArgs['groupBy'],
            'limit'     => empty($aArgs['limit']) ? 0 : $aArgs['limit']
        ]);

        return $attachments;
    }

    public static function getById(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id']);
        ValidatorModel::intVal($aArgs, ['id']);
        ValidatorModel::arrayType($aArgs, ['select']);

        $attachment = DatabaseModel::select([
            'select'    => empty($aArgs['select']) ? ['*'] : $aArgs['select'],
            'table'     => ['res_attachments'],
            'where'     => ['res_id = ?'],
            'data'      => [$aArgs['id']],
        ]);

        if (empty($attachment[0])) {
            return [];
        }

        return $attachment[0];
    }

    public static function getAttachmentToSend(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['ids']);
        ValidatorModel::arrayType($aArgs, ['select', 'orderBy', 'ids']);

        $aAttachments = DatabaseModel::select([
            'select'    => empty($aArgs['select']) ? ['max(relation) as relation', 'res_id_master', 'title', 'res_id', 'identifier', 'dest_address_id'] : $aArgs['select'],
            'table'     => ['res_attachments'],
            'where'     => ['res_id_master in (?)', 'status not in (?)', 'attachment_type not in (?)', 'in_send_attach = TRUE'],
            'data'      => [$aArgs['ids'], ['OBS', 'DEL', 'TMP', 'FRZ'], 'print_folder'],
            'groupBy'   => ['res_id_master', 'title', 'res_id', 'identifier', 'dest_address_id'],
            'order_by'  => empty($aArgs['orderBy']) ? [] : $aArgs['orderBy']
        ]);

        return $aAttachments;
    }

    public static function getListByResIdMaster(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['resId', 'login']);
        ValidatorModel::intVal($aArgs, ['resId', 'limit']);
        ValidatorModel::stringType($aArgs, ['login']);
        ValidatorModel::arrayType($aArgs, ['excludeAttachmentTypes']);

        $aAttachments = DatabaseModel::select([
            'select'    => [
                'res_id', 'identifier', 'title', 'creation_date', 'modification_date', 'validation_date as return_date', 'effective_date as real_return_date',
                'relation', 'status', 'attachment_type', 'in_signature_book', 'in_send_attach'
            ],
            'table'     => ['res_attachments'],
            'where'     => ['res_id_master = ?', 'res_attachments.status not in (?)', 'attachment_type not in (?)', '((res_attachments.status = ? AND typist = ?) OR res_attachments.status != ?)'],
            'data'      => [$aArgs['resId'], ['OBS', 'DEL'], $aArgs['excludeAttachmentTypes'], 'TMP', $aArgs['login'], 'TMP'],
            'order_by'  => empty($aArgs['orderBy']) ? [] : $aArgs['orderBy'],
            'limit'     => empty($aArgs['limit']) ? null : $aArgs['limit']
        ]);

        return $aAttachments;
    }

    public static function create(array $args)
    {
        ValidatorModel::notEmpty($args, ['format', 'typist', 'creation_date', 'docserver_id', 'path', 'filename', 'fingerprint', 'filesize', 'status', 'relation']);
        ValidatorModel::stringType($args, ['format', 'typist', 'creation_date', 'docserver_id', 'path', 'filename', 'fingerprint', 'status']);
        ValidatorModel::intVal($args, ['filesize', 'relation']);

        $nextSequenceId = DatabaseModel::getNextSequenceValue(['sequenceId' => 'res_attachment_res_id_seq']);
        $args['res_id'] = $nextSequenceId;

        DatabaseModel::insert([
            'table'         => 'res_attachments',
            'columnsValues' => $args
        ]);

        return $nextSequenceId;
    }

    public static function update(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['set', 'where', 'data']);
        ValidatorModel::arrayType($aArgs, ['set', 'where', 'data']);

        DatabaseModel::update([
            'table' => 'res_attachments',
            'set'   => $aArgs['set'],
            'where' => $aArgs['where'],
            'data'  => $aArgs['data']
        ]);

        return true;
    }

    public static function getAttachmentsTypesByXML()
    {
        static $types;

        if (!empty($types)) {
            return $types;
        }

        $types = [];

        $loadedXml = CoreConfigModel::getXmlLoaded(['path' => 'apps/maarch_entreprise/xml/entreprise.xml']);
        if ($loadedXml) {
            $attachmentTypesXML = $loadedXml->attachment_types;
            if (count($attachmentTypesXML) > 0) {
                foreach ($attachmentTypesXML->type as $value) {
                    $label = defined((string) $value->label) ? constant((string) $value->label) : (string) $value->label;
                    $types[(string) $value->id] = [
                        'label'     => $label,
                        'icon'      => (string)$value['icon'],
                        'sign'      => (empty($value['sign']) || (string)$value['sign'] == 'true') ? true : false,
                        'chrono'    => (empty($value['with_chrono']) || (string)$value['with_chrono'] == 'true') ? true : false,
                        'show'      => (empty($value->attributes()->show) || (string)$value->attributes()->show == 'true') ? true : false
                    ];
                }
            }
        }

        return $types;
    }

    public static function unsignAttachment(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['resId']);
        ValidatorModel::intVal($aArgs, ['resId']);

        DatabaseModel::update([
            'table'     => 'res_attachments',
            'set'       => ['status' => 'A_TRA', 'signatory_user_serial_id' => null],
            'where'     => ['res_id = ?'],
            'data'      => [$aArgs['resId']]
        ]);

        DatabaseModel::update([
            'table'     => 'res_attachments',
            'set'       => ['status' => 'DEL'],
            'where'     => ['origin = ?', 'status != ?'],
            'data'      => ["{$aArgs['resId']},res_attachments", 'DEL']
        ]);

        return true;
    }

    public static function freezeAttachment(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['resId', 'externalId']);
        ValidatorModel::intType($aArgs, ['resId']);

        $aAttachment = DatabaseModel::select([
            'select'    => ['external_id'],
            'table'     => ['res_attachments'],
            'where'     => ['res_id = ?'],
            'data'      => [$aArgs['resId']],
        ]);

        $externalId = json_decode($aAttachment[0]['external_id'], true);
        $externalId['signatureBookId'] = empty($aArgs['externalId']) ? null : $aArgs['externalId'];

        DatabaseModel::update([
            'table'     => 'res_attachments',
            'set'       => ['status' => 'FRZ', 'external_id' => json_encode($externalId)],
            'where'     => ['res_id = ?'],
            'data'      => [$aArgs['resId']]
        ]);

        return true;
    }

    public static function setInSignatureBook(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id']);
        ValidatorModel::intVal($aArgs, ['id']);
        ValidatorModel::boolType($aArgs, ['inSignatureBook']);

        if ($aArgs['inSignatureBook']) {
            $aArgs['inSignatureBook'] =  'true';
        } else {
            $aArgs['inSignatureBook'] =  'false';
        }

        DatabaseModel::update([
            'table'     => 'res_attachments',
            'set'       => [
                'in_signature_book'   => $aArgs['inSignatureBook']
            ],
            'where'     => ['res_id = ?'],
            'data'      => [$aArgs['id']],
        ]);

        return true;
    }

    public static function setInSendAttachment(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id']);
        ValidatorModel::intVal($aArgs, ['id']);
        ValidatorModel::boolType($aArgs, ['inSendAttachment']);

        if ($aArgs['inSendAttachment']) {
            $aArgs['inSendAttachment'] =  'true';
        } else {
            $aArgs['inSendAttachment'] =  'false';
        }

        DatabaseModel::update([
            'table'     => 'res_attachments',
            'set'       => [
                'in_send_attach'   => $aArgs['inSendAttachment']
            ],
            'where'     => ['res_id = ?'],
            'data'      => [$aArgs['id']],
        ]);

        return true;
    }

    public static function hasAttachmentsSignedForUserById(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'user_serial_id']);
        ValidatorModel::intVal($aArgs, ['id', 'user_serial_id']);

        $attachment = DatabaseModel::select([
            'select'    => ['res_id_master'],
            'table'     => ['res_attachments'],
            'where'     => ['res_id = ?'],
            'data'      => [$aArgs['id']],
        ]);

        $attachments = DatabaseModel::select([
            'select'    => ['res_id_master'],
            'table'     => ['res_attachments'],
            'where'     => ['res_id_master = ?', 'signatory_user_serial_id = ?'],
            'data'      => [$attachment[0]['res_id_master'], $aArgs['user_serial_id']],
        ]);

        if (empty($attachments)) {
            return false;
        }

        return true;
    }

    public static function delete(array $args)
    {
        ValidatorModel::notEmpty($args, ['where', 'data']);
        ValidatorModel::arrayType($args, ['where', 'data']);

        DatabaseModel::update([
            'table' => 'res_attachments',
            'set'   => [
                'status'    => 'DEL'
            ],
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }
}
